@@ -3,9 +3,26 @@ package Data::Mining::Apriori;
 use 5.010001;
 use strict;
 use warnings;
+use Algorithm::Combinatorics qw(subsets variations);
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.10;
+use Exporter;
+our @ISA = qw(Exporter);
+our %EXPORT_TAGS = ('all'=>[qw(
+	validate_data
+	quantity_possible_rules
+	generate_rules
+	association_rules
+	stop
+	output
+	data_persistence
+	file
+	excel
+)]);
+our @EXPORT_OK = (@{$EXPORT_TAGS{'all'}});
+our @EXPORT = qw();
+
+our $VERSION = 0.11;
 
 my$self;
 
@@ -17,7 +34,7 @@ sub new{
 	my $type = shift;
 	my $class = ref($type)||$type;
 	$self = {
-		totalTransactions => undef,
+		totalTransactions => 0,
 		minSupport => 1,
 		minConfidence => undef,
 		minLift => undef,
@@ -33,7 +50,7 @@ sub new{
 		associationRules => undef,
 		implications => undef,
 		largeItemsetSize => 2,
-		rule => 1
+		rule => 0
 	};
 	bless($self,$class);
 	return $self;
@@ -46,121 +63,106 @@ sub validate_data{
 		or die("Error: \$apriori->{itemsKeyTransactions} is not defined!\n");
 }
 
+sub insert_key_items_transaction{
+	(scalar(@_)==2)
+		or die("Error: \$apriori->insert_key_items_transaction(\\\@items) missing parameter key items to array reference!\n");
+	(ref($_[1]) eq "ARRAY")
+		or die("Error: \$apriori->insert_key_items_transaction(\\\@items) parameter key items is not an array reference!\n");
+	my@items=sort(@{$_[1]});
+	my @itemsets=subsets(\@items);
+	foreach my$itemset(@itemsets){
+		$self->{itemsKeyTransactions}{"@{$itemset}"}++;
+	}
+	$self->{totalTransactions}++;
+}
+
+sub input_data_file{
+	(scalar(@_)==3)
+		or die("Error: \$apriori->input_data_file(\"datafile.txt\",\",\") missing parameters path to data file and/or item separator!\n");
+	my$file=$_[1];
+	my$separator=$_[2];
+	open(FILE,"<$file")
+		or die("Error: $!");
+	while(my$line=<FILE>){
+		$line=~s/\r\n|\n//;
+		my@items=split($separator,$line);
+		if(scalar(@items)>=2){
+			$self->insert_key_items_transaction(\@items);
+		}
+	}
+	close(FILE);
+}
+
 sub quantity_possible_rules{
 	$self->validate_data;
-	my $quantityItems = scalar(keys(%{$self->{itemsKeyDescription}}));
-	return ((3**$quantityItems)-(2**($quantityItems+1))+1);
+	return ((3**scalar(keys(%{$self->{itemsKeyDescription}})))-(2**(scalar(keys(%{$self->{itemsKeyDescription}}))+1))+1);
 }
 
 sub generate_rules{
 	$self->validate_data;
 	my @largeItemsetSizeOne = $self->data_persistence;
 	if($self->{messages}){
-		my $quantityItems = scalar(keys(%{$self->{itemsKeyDescription}}));
-		print "\n$quantityItems items, ${\$self->quantity_possible_rules} possible rules";
+		print "\n${\scalar(keys(%{$self->{itemsKeyDescription}}))} items, ${\$self->quantity_possible_rules} possible rules";
 	}
-	my @transactions;
-	foreach my$item(keys(%{$self->{itemsKeyDescription}})){
-		foreach my$transaction(@{$self->{itemsKeyTransactions}{$item}}){
-			if(!($transaction~~@transactions)){
-				push@transactions,$transaction;
-			}
-		}
-	}
-	$self->{totalTransactions} = scalar(@transactions);
-	my @frequentItemset = grep{((scalar(@{$self->{itemsKeyTransactions}{$_}})/$self->{totalTransactions})*100)>=$self->{minSupport}}@largeItemsetSizeOne;
+	my @frequentItemset = grep{(($self->{itemsKeyTransactions}{$_}/$self->{totalTransactions})*100)>=$self->{minSupport}}@largeItemsetSizeOne;
 	$self->association_rules(\@frequentItemset);
 }
 
 sub association_rules{
-	my @largeItemsetSizeOne = @{$_[1]};
+	my @largeItemset = @{$_[1]};
+	my @variations = variations(\@largeItemset,$self->{largeItemsetSize});
 	my @frequentItemset;
 	if($self->{messages}){
-		print "\nLarge itemset size ${\$self->{largeItemsetSize}}, ${\scalar(@largeItemsetSizeOne)} items ";
+		print "\nLarge itemset size $self->{largeItemsetSize}, ${\scalar(@largeItemset)} items ";
 		print "\nProcessing... ";
 	}
-	for(my$largeItemsetSize=2;$largeItemsetSize<=$self->{largeItemsetSize};$largeItemsetSize++){
-		my $code=<<'CODE';
-my @candidateItemset;
-$self->{implications}={};
-for(my$k=0;$k<=$#largeItemsetSizeOne;$k++){
-	push @candidateItemset,$largeItemsetSizeOne[$k];
-CODE
-		for(my$i=2;$i<=$largeItemsetSize;$i++){
-			$code.=<<'CODE';
-for(my$l=0;$l<=$#largeItemsetSizeOne;$l++){
-	if($largeItemsetSizeOne[$l] ~~ @candidateItemset){
-		next;
-	}
-	push @candidateItemset,$largeItemsetSizeOne[$l];
-CODE
-		}
-		$code.=<<'CODE';
-for(my$antecedentSize=0;$antecedentSize<$#candidateItemset;$antecedentSize++){
-	my @antecedent;
-	my @consequent;
-	for(my$m=0;$m<=$antecedentSize;$m++){
-		push @antecedent,$candidateItemset[$m];
-	}
-	@antecedent = sort @antecedent;
-	@consequent = grep{!($_~~@antecedent)}@candidateItemset;
-	@consequent = sort @consequent;
-	next if("@consequent" ~~ @{$self->{implications}{"@antecedent"}});
-	push @{$self->{implications}{"@antecedent"}},"@consequent";
-	my @transactions = @{$self->{itemsKeyTransactions}{$antecedent[0]}};
-	foreach my$item(@antecedent){
-		@transactions = grep{$_~~@transactions}@{$self->{itemsKeyTransactions}{$item}};
-	}
-	my $supportAntecedent = scalar(@transactions);
-	next if($supportAntecedent == 0);
-	foreach my$item(@consequent){
-		@transactions = grep{$_~~@transactions}@{$self->{itemsKeyTransactions}{$item}};
-	}
-	my $supportConsequent = scalar(@transactions);
-	next if($supportConsequent == 0);
-	my $support = sprintf("%.2f",(($supportConsequent/$self->{totalTransactions})*100));
-	next if($support < $self->{minSupport});
-	my $confidence = sprintf("%.2f",(($supportConsequent/$supportAntecedent)*100));
-	next if(defined $self->{minConfidence} && $confidence < $self->{minConfidence});
-	my $lift = sprintf("%.2f",($support/($supportAntecedent*$supportConsequent)));
-	next if(defined $self->{minLift} && $lift < $self->{minLift});
-	my $leverage = sprintf("%.2f",($support-($supportAntecedent*$supportConsequent)));
-	next if(defined $self->{minLeverage} && $leverage < $self->{minLeverage});	
-	my $conviction = sprintf("%.2f",((1-$supportAntecedent)/(1-$confidence)));
-	next if(defined $self->{minConviction} && $conviction < $self->{minConviction});
-	my $coverage = sprintf("%.2f",(($supportAntecedent/$self->{totalTransactions})*100));
-	next if(defined $self->{minCoverage} && $coverage < $self->{minCoverage});
-	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence, $lift, $leverage, $conviction, $coverage];
-	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
-	my@items=grep{!($_~~@frequentItemset)}@antecedent;
-	push @frequentItemset,@items;
-	@items=grep{!($_~~@frequentItemset)}@consequent;
-	push @frequentItemset,@items;
-	return if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
-	$self->{rule}++;
-}
-CODE
-		for(my$j=2;$j<=$largeItemsetSize;$j++){
-			$code.=<<'CODE';
-	pop @candidateItemset;
-}
-CODE
-		}
-		$code.=<<'CODE';
-	pop @candidateItemset;
-}
-CODE
-		if($largeItemsetSize==$self->{largeItemsetSize}){
-			eval $code;
-			die("Error: $@\n") if $@;
-			if($self->{messages}){
-				print "\nFrequent itemset: { @frequentItemset }, ${\scalar(@frequentItemset)} items ";
-			}
-			if(defined $self->{associationRules}){
-				@{$self->{frequentItemset}}=@frequentItemset;
-				$self->output;
+	foreach my$variation(@variations){
+		my@candidateItemset=@{$variation};
+		my@antecedent;
+		my@consequent;
+		for(my$antecedentSize=0;$antecedentSize<$#candidateItemset;$antecedentSize++){
+			push@antecedent,$candidateItemset[$antecedentSize];
+			@consequent=();
+			for(my$consequentSize=($antecedentSize+1);$consequentSize<=$#candidateItemset;$consequentSize++){
+				push@consequent,$candidateItemset[$consequentSize];
 			}
+			@antecedent=sort@antecedent;
+			@consequent=sort@consequent;
+			next if("@consequent" ~~ @{$self->{implications}{"@antecedent"}});
+			push @{$self->{implications}{"@antecedent"}},"@consequent";
+			my@implication;
+			push@implication,@antecedent,@consequent;
+			@implication=sort(@implication);
+			next unless $self->{itemsKeyTransactions}{"@antecedent"};
+			my$supportAntecedent=$self->{itemsKeyTransactions}{"@antecedent"};
+			next unless $self->{itemsKeyTransactions}{"@implication"};
+			my$supportConsequent=$self->{itemsKeyTransactions}{"@implication"};
+			my $support = sprintf("%.2f",(($supportConsequent/$self->{totalTransactions})*100));
+			next if($support<$self->{minSupport});
+			my $confidence = sprintf("%.2f",(($supportConsequent/$supportAntecedent)*100));
+			next if(defined $self->{minConfidence} && $confidence < $self->{minConfidence});
+			my $lift = sprintf("%.2f",($support/($supportAntecedent*$supportConsequent)));
+			next if(defined $self->{minLift} && $lift < $self->{minLift});
+			my $leverage = sprintf("%.2f",($support-($supportAntecedent*$supportConsequent)));
+			next if(defined $self->{minLeverage} && $leverage < $self->{minLeverage});	
+			my $conviction = sprintf("%.2f",((1-$supportAntecedent)/(1-$confidence)));
+			next if(defined $self->{minConviction} && $conviction < $self->{minConviction});
+			my $coverage = sprintf("%.2f",(($supportAntecedent/$self->{totalTransactions})*100));
+			next if(defined $self->{minCoverage} && $coverage < $self->{minCoverage});
+			$self->{rule}++;
+			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence, $lift, $leverage, $conviction, $coverage];
+			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
+			my@items=grep{!($_~~@frequentItemset)}@implication;
+			push@frequentItemset,@items;
 		}
+		last if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
+	}
+	if($self->{messages}){
+		print "\nFrequent itemset: { @frequentItemset }, ${\scalar(@frequentItemset)} items ";
+	}
+	if(defined $self->{associationRules}){
+		@{$self->{frequentItemset}}=@frequentItemset;
+		$self->output;
 	}
 	return if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
 	if(scalar(@frequentItemset)>=($self->{largeItemsetSize}+1)){
@@ -237,7 +239,8 @@ sub file{
 	if($self->{messages}){
 		print "\nExporting to file \"output_large_itemset_size_$self->{largeItemsetSize}.txt\"... ";
 	}
-	open(FILE,">output_large_itemset_size_$self->{largeItemsetSize}.txt");
+	open(FILE,">output_large_itemset_size_$self->{largeItemsetSize}.txt")
+		or die("\nError: \"output_large_itemset_size_$self->{largeItemsetSize}.txt\" $!");
 	print FILE "Rules\tSupport %";
 	print FILE "\tConfidence %" if defined $self->{minConfidence};
 	print FILE "\tLift" if defined $self->{minLift};
@@ -444,7 +447,8 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{minCoverage}=0; # The minimum coverage(percent, optional)
 
-	$apriori->{output}=1; # The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
+	$apriori->{output}=1;
+	# The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
 
 	$apriori->{messages}=1; # A value boolean to display the messages(optional)
 
@@ -452,16 +456,42 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	$apriori->{itemsKeyDescription}{102}='BREAD';
 	$apriori->{itemsKeyDescription}{'103'}='CEREAL';
 
-	@{$apriori->{itemsKeyTransactions}{'101'}}=('1101',1103,'1104',1105,'1106',1107,'1109',1110);
-	# Reference to array, to add the transactions of each item per key
-	@{$apriori->{itemsKeyTransactions}{102}}=('1102',1103,'1104',1105,'1106',1107,1108,'1109',1110);
-	@{$apriori->{itemsKeyTransactions}{'103'}}=('1101',1102,1103,'1104','1106',1107,1108,'1109',1110);
+	my@items=(103,101);
+	$apriori->insert_key_items_transaction(\@items); # Insert key items per transaction
+	$apriori->insert_key_items_transaction([103,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
+	$apriori->insert_key_items_transaction([101,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
+	$apriori->insert_key_items_transaction([103,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
+	$apriori->insert_key_items_transaction([103,101,102]);
 
-	print "\n${\$apriori->quantity_possible_rules}"; # Prints the quantity of possible rules
+	# or from a data file
+
+	# $apriori->input_data_file("datafile.txt",",");
+	# Insert items per line(transaction), accepts the arguments of path to data file and item separator
+
+	# file contents
+
+	103,101
+	103,102
+	103,101,102
+	103,101,102
+	101,102
+	103,101,102
+	103,101,102
+	103,102
+	103,101,102
+	103,101,102
+
+	print "\n${\$apriori->quantity_possible_rules}"; # Show the quantity of possible rules
 
 	$apriori->{limitRules}=10; # The limit of rules
 
-	$apriori->generate_rules; # Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage or limit of rules
+	$apriori->generate_rules;
+	# Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage or limit of rules
 
 	print "\n@{$apriori->{frequentItemset}}\n"; # Show frequent items
 
@@ -509,8 +539,6 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	R8	60,00	85,71	1,43	18,00	0,07	70,00
 	R9	60,00	75,00	1,25	12,00	0,09	80,00
 	R10	60,00	75,00	1,25	12,00	0,09	80,00
-	R11	60,00	85,71	1,43	18,00	0,07	70,00
-	R12	60,00	66,67	1,11	6,00	0,12	90,00
 
 	Rule R7: { 103 } => { 101, 102 }
 	Support: 60,00 %
@@ -541,28 +569,28 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# or from a database
 
 	# CREATE TABLE dimension_product(
-	# 	product_key INTEGER NOT NULL PRIMARY KEY,
-	# 	product_alternate_key INTEGER NOT NULL,
-	# 	product_name TEXT NOT NULL,
-	# 	price REAL NOT NULL
-	#	// ...
+		# product_key INTEGER NOT NULL PRIMARY KEY,
+		# product_alternate_key INTEGER NOT NULL,
+		# product_name TEXT NOT NULL,
+		# price REAL NOT NULL
+		# -- ...
 	# );
-	#
+
 	# INSERT INTO dimension_product VALUES(1,101,'MILK',10.00);
 	# INSERT INTO dimension_product VALUES(2,102,'BREAD',10.00);
 	# INSERT INTO dimension_product VALUES(3,103,'CEREAL',10.00);
-	# // ...
-	# 
+	# -- ...
+
 	# CREATE TABLE fact_sales(
-	# 	sales_order_number INTEGER NOT NULL,
-	# 	sales_order_line_number INTEGER NOT NULL,
-	# 	product_key INTEGER NOT NULL,
-	# 	quantity INTEGER NOT NULL,
-	#	// ...
-	# 	PRIMARY KEY(sales_order_number, sales_order_line_number),
-	# 	FOREIGN KEY(product_key) REFERENCES dimension_product(product_key)
+		# sales_order_number INTEGER NOT NULL,
+		# sales_order_line_number INTEGER NOT NULL,
+		# product_key INTEGER NOT NULL,
+		# quantity INTEGER NOT NULL,
+		# -- ...
+		# PRIMARY KEY(sales_order_number, sales_order_line_number),
+		# FOREIGN KEY(product_key) REFERENCES dimension_product(product_key)
 	# );
-	#
+
 	# INSERT INTO fact_sales VALUES(1101,1,3,1);
 	# INSERT INTO fact_sales VALUES(1101,2,1,1);
 	# INSERT INTO fact_sales VALUES(1102,1,3,1);
@@ -589,23 +617,12 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# INSERT INTO fact_sales VALUES(1110,1,1,1);
 	# INSERT INTO fact_sales VALUES(1110,2,2,1);
 	# INSERT INTO fact_sales VALUES(1110,3,3,1);
-	#//...
+	# -- ...
 
 	use DBD::SQLite;
 	use Data::Mining::Apriori;
 
-	my $db = DBI->connect('dbi:SQLite:dbname=DW.db','','');
-
-	my $sql = q~
-	SELECT COUNT(DISTINCT(sales_order_number)) FROM fact_sales
-	/* WHERE ... */
-	~;
-
-	my $query = $db->prepare($sql);
-	$query->execute;
-	my $totalTransactions = $query->fetchrow;
-
-	$apriori = new Data::Mining::Apriori;
+	my $apriori = new Data::Mining::Apriori;
 
 	$apriori->{minSupport}=1.55;
 
@@ -623,34 +640,37 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{messages}=1;
 
-	$sql = qq~
-	SELECT dp.product_alternate_key, dp.product_name
+	my $db = DBI->connect('dbi:SQLite:dbname=DW.db','','');
+
+	my$sql = qq~
+	SELECT DISTINCT(fs.sales_order_number)
 	FROM dimension_product dp
 	JOIN fact_sales fs ON
 	dp.product_key = fs.product_key
-	/* WHERE ... */
+	-- WHERE ...
 	~;
 
-	$query = $db->prepare($sql);
+	my$query = $db->prepare($sql);
 	$query->execute;
-	while(my($key,$description)=$query->fetchrow_array){
-		$apriori->{itemsKeyDescription}{$key}=$description;
-	}
+	my$transactions=$query->fetchall_arrayref;
 
-	foreach my$key(keys(%{$apriori->{itemsKeyDescription}})){
+	foreach my$transaction(@$transactions){
 		$sql = qq~
-		SELECT DISTINCT(fs.sales_order_number)
-		FROM dimension_product dp
-		JOIN fact_sales fs ON
-		dp.product_key = fs.product_key
-		WHERE dp.product_alternate_key = $key
-		/* AND ... */
+			SELECT dp.product_alternate_key, dp.product_name
+			FROM dimension_product dp
+			JOIN fact_sales fs ON
+			dp.product_key = fs.product_key
+			WHERE fs.sales_order_number = $$transaction[0];
+			-- AND ...
 		~;
 		$query = $db->prepare($sql);
 		$query->execute;
-		while(my$transaction=$query->fetchrow){
-			push @{$apriori->{itemsKeyTransactions}{$key}},$transaction;
+		my@items;
+		while(my($key,$description)=$query->fetchrow){
+			$apriori->{itemsKeyDescription}{$key}=$description;
+			push@items,$key;
 		}
+		$apriori->insert_key_items_transaction(\@items);
 	}
 
 	print "\n${\$apriori->quantity_possible_rules}";
@@ -757,6 +777,28 @@ A data structure to store the name of the rule, key items, implication, support,
 
 Creates a new instance of Data::Mining::Apriori.
 
+=head2 insert_key_items_transaction(\@items)
+
+Insert key items per transaction. Accepts the following arguments:
+
+=over 4
+
+=item * An array reference to key items.
+
+=back
+
+=head2 input_data_file("datafile.txt",",")
+
+Insert items per line(transaction). Accepts the following arguments:
+
+=over 4
+
+=item * Data file;
+
+=item * Item separator.
+
+=back
+
 =head2 quantity_possible_rules
 
 Returns the quantity of possible rules.
@@ -767,7 +809,7 @@ Generate association rules until no set of items meets the minimum support, conf
 
 =head2 association_rules
 
-Generates association rules by size set of larger itemsets.
+Generate association rules by size of large itemsets.
 
 =head1 AUTHOR
 
